@@ -12,26 +12,30 @@
 
 package scala.collection.concurrent;
 
+import java.lang.Object;
+
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
 
 abstract class INodeBase<K, V> extends BasicNode {
 
-    @SuppressWarnings("rawtypes")
-    public static final AtomicReferenceFieldUpdater<INodeBase, MainNode> updater =
-            AtomicReferenceFieldUpdater.newUpdater(INodeBase.class, MainNode.class, "mainnode");
+    @SuppressWarnings("unchecked")
+    public static final AtomicReferenceFieldUpdater<INodeBase<?, ?>, MainNode<?, ?>> updater =
+            AtomicReferenceFieldUpdater.newUpdater((Class<INodeBase<?, ?>>) (Class<?>) INodeBase.class, (Class<MainNode<?, ?>>) (Class<?>) MainNode.class, "mainnode");
 
-    public static final Object RESTART = new Object();
+    static final Object RESTART = new Object();
+
+    static final Object NO_SUCH_ELEMENT_SENTINEL = new Object();
 
     public volatile MainNode<K, V> mainnode = null;
 
     public final Gen gen;
 
     public INodeBase(Gen generation) {
-	gen = generation;
+        gen = generation;
     }
 
     public BasicNode prev() {
 	return null;
     }
 
-}
\ No newline at end of file
+}
