@@ -12,20 +12,22 @@
 
 package scala.collection.concurrent;
 
+import java.lang.Object;
+
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
 
 abstract class MainNode<K, V> extends BasicNode {
 
-    @SuppressWarnings("rawtypes")
-    public static final AtomicReferenceFieldUpdater<MainNode, MainNode> updater =
-            AtomicReferenceFieldUpdater.newUpdater(MainNode.class, MainNode.class, "prev");
+    @SuppressWarnings("unchecked")
+    public static final AtomicReferenceFieldUpdater<MainNode<?, ?>, MainNode<?, ?>> updater =
+            AtomicReferenceFieldUpdater.newUpdater((Class<MainNode<?, ?>>) (Class<?>) MainNode.class, (Class<MainNode<?, ?>>) (Class<?>) MainNode.class, "prev");
 
     public volatile MainNode<K, V> prev = null;
 
     public abstract int cachedSize(Object ct);
 
     public boolean CAS_PREV(MainNode<K, V> oldval, MainNode<K, V> nval) {
-	return updater.compareAndSet(this, oldval, nval);
+        return updater.compareAndSet(this, oldval, nval);
     }
 
     public void WRITE_PREV(MainNode<K, V> nval) {
@@ -37,7 +39,7 @@ abstract class MainNode<K, V> extends BasicNode {
     // irregardless of whether there are concurrent ARFU updates
     @Deprecated @SuppressWarnings("unchecked")
     public MainNode<K, V> READ_PREV() {
-	return updater.get(this);
+        return (MainNode<K, V>) updater.get(this);
     }
 
-}
\ No newline at end of file
+}
