@@ -390,19 +390,19 @@ trait Internals { self: Universe =>
 
       /** @see [[TreeDecorator]] */
       class TreeDecoratorApi[T <: Tree](val tree: T) {
-        /** @see [[internal.freeTerms]] */
+        /** @see [[InternalApi.freeTerms]] */
         def freeTerms: List[FreeTermSymbol] = internal.freeTerms(tree)
 
-        /** @see [[internal.freeTypes]] */
+        /** @see [[InternalApi.freeTypes]] */
         def freeTypes: List[FreeTypeSymbol] = internal.freeTypes(tree)
 
-        /** @see [[internal.substituteSymbols]] */
+        /** @see [[InternalApi.substituteSymbols]] */
         def substituteSymbols(from: List[Symbol], to: List[Symbol]): Tree = internal.substituteSymbols(tree, from, to)
 
-        /** @see [[internal.substituteTypes]] */
+        /** @see [[InternalApi.substituteTypes]] */
         def substituteTypes(from: List[Symbol], to: List[Type]): Tree = internal.substituteTypes(tree, from, to)
 
-        /** @see [[internal.substituteThis]] */
+        /** @see [[InternalApi.substituteThis]] */
         def substituteThis(clazz: Symbol, to: => Tree): Tree = internal.substituteThis(tree, clazz, to)
       }
 
@@ -414,49 +414,49 @@ trait Internals { self: Universe =>
 
       /** @see [[SymbolDecorator]] */
       class SymbolDecoratorApi[T <: Symbol](val symbol: T) {
-        /** @see [[internal.isFreeTerm]] */
+        /** @see [[InternalApi.isFreeTerm]] */
         def isFreeTerm: Boolean = internal.isFreeTerm(symbol)
 
-        /** @see [[internal.asFreeTerm]] */
+        /** @see [[InternalApi.asFreeTerm]] */
         def asFreeTerm: FreeTermSymbol = internal.asFreeTerm(symbol)
 
-        /** @see [[internal.isFreeType]] */
+        /** @see [[InternalApi.isFreeType]] */
         def isFreeType: Boolean = internal.isFreeType(symbol)
 
-        /** @see [[internal.asFreeType]] */
+        /** @see [[InternalApi.asFreeType]] */
         def asFreeType: FreeTypeSymbol = internal.asFreeType(symbol)
 
-        /** @see [[internal.newTermSymbol]] */
+        /** @see [[InternalApi.newTermSymbol]] */
         def newTermSymbol(name: TermName, pos: Position = NoPosition, flags: FlagSet = NoFlags): TermSymbol = internal.newTermSymbol(symbol, name, pos, flags)
 
-        /** @see [[internal.newModuleAndClassSymbol]] */
+        /** @see [[InternalApi.newModuleAndClassSymbol]] */
         def newModuleAndClassSymbol(name: Name, pos: Position = NoPosition, flags: FlagSet = NoFlags): (ModuleSymbol, ClassSymbol) = internal.newModuleAndClassSymbol(symbol, name, pos, flags)
 
-        /** @see [[internal.newMethodSymbol]] */
+        /** @see [[InternalApi.newMethodSymbol]] */
         def newMethodSymbol(name: TermName, pos: Position = NoPosition, flags: FlagSet = NoFlags): MethodSymbol = internal.newMethodSymbol(symbol, name, pos, flags)
 
-        /** @see [[internal.newTypeSymbol]] */
+        /** @see [[InternalApi.newTypeSymbol]] */
         def newTypeSymbol(name: TypeName, pos: Position = NoPosition, flags: FlagSet = NoFlags): TypeSymbol = internal.newTypeSymbol(symbol, name, pos, flags)
 
-        /** @see [[internal.newClassSymbol]] */
+        /** @see [[InternalApi.newClassSymbol]] */
         def newClassSymbol(name: TypeName, pos: Position = NoPosition, flags: FlagSet = NoFlags): ClassSymbol = internal.newClassSymbol(symbol, name, pos, flags)
 
-        /** @see [[internal.isErroneous]] */
+        /** @see [[InternalApi.isErroneous]] */
         def isErroneous: Boolean = internal.isErroneous(symbol)
 
-        /** @see [[internal.isSkolem]] */
+        /** @see [[InternalApi.isSkolem]] */
         def isSkolem: Boolean = internal.isSkolem(symbol)
 
-        /** @see [[internal.deSkolemize]] */
+        /** @see [[InternalApi.deSkolemize]] */
         def deSkolemize: Symbol = internal.deSkolemize(symbol)
 
-        /** @see [[internal.initialize]] */
+        /** @see [[InternalApi.initialize]] */
         def initialize: T = internal.initialize(symbol)
 
-        /** @see [[internal.fullyInitialize]] */
+        /** @see [[InternalApi.fullyInitialize]] */
         def fullyInitialize: T = internal.fullyInitialize(symbol)
 
-        /** @see [[internal.flags]] */
+        /** @see [[InternalApi.flags]] */
         def flags: FlagSet = internal.flags(symbol)
       }
 
@@ -468,7 +468,7 @@ trait Internals { self: Universe =>
 
       /** @see [[TypeDecorator]] */
       implicit class TypeDecoratorApi[T <: Type](val tp: T) {
-        /** @see [[internal.fullyInitialize]] */
+        /** @see [[InternalApi.fullyInitialize]] */
         def fullyInitialize: T = internal.fullyInitialize(tp)
       }
     }
@@ -746,7 +746,7 @@ trait Internals { self: Universe =>
 
     trait SyntacticFilterExtractor {
       def apply(test: Tree): Tree
-      def unapply(tree: Tree): Option[(Tree)]
+      def unapply(tree: Tree): Option[Tree]
     }
 
     val SyntacticEmptyTypeTree: SyntacticEmptyTypeTreeExtractor
@@ -853,12 +853,6 @@ trait Internals { self: Universe =>
     }
   }
 
-  @deprecated("use `internal.reificationSupport` instead", "2.11.0")
-  val build: ReificationSupportApi
-
-  @deprecated("use `internal.ReificationSupportApi` instead", "2.11.0")
-  type BuildApi = ReificationSupportApi
-
   /** This trait provides support for importers, a facility to migrate reflection artifacts between universes.
    * ''Note: this trait should typically be used only rarely.''
    *
@@ -1068,11 +1062,13 @@ trait Internals { self: Universe =>
    *  a single `compat._` import will fix things for you.
    *  @group Internal
    */
+  @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
   val compat: Compat
 
   /** @see [[compat]]
    *  @group Internal
    */
+  @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
   type Compat <: CompatApi
 
   /** Presence of an implicit value of this type in scope
@@ -1080,171 +1076,28 @@ trait Internals { self: Universe =>
    *  @group Internal
    */
   @scala.annotation.implicitNotFound("This method has been removed from the public API. Import compat._ or migrate away.")
+  @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
   class CompatToken
 
   /** @see [[compat]]
    *  @group Internal
    */
+  @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
   trait CompatApi {
     /** @see [[CompatToken]] */
+    @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
     implicit val token = new CompatToken
 
-    /** @see [[InternalApi.typeTagToManifest]] */
-    @deprecated("use `internal.typeTagToManifest` instead", "2.11.0")
-    def typeTagToManifest[T: ClassTag](mirror: Any, tag: Universe#TypeTag[T]): Manifest[T] =
-      internal.typeTagToManifest(mirror, tag)
-
-    /** @see [[InternalApi.manifestToTypeTag]] */
-    @deprecated("use `internal.manifestToTypeTag` instead", "2.11.0")
-    def manifestToTypeTag[T](mirror: Any, manifest: Manifest[T]): Universe#TypeTag[T] =
-      internal.manifestToTypeTag(mirror, manifest)
-
-    /** @see [[InternalApi.newScopeWith]] */
-    @deprecated("use `internal.newScopeWith` instead", "2.11.0")
-    def newScopeWith(elems: Symbol*): Scope =
-      internal.newScopeWith(elems: _*)
-
     /** Scala 2.10 compatibility enrichments for BuildApi. */
-    implicit class CompatibleBuildApi(api: BuildApi) {
-      /** @see [[BuildApi.setInfo]] */
-      @deprecated("use `internal.reificationSupport.setInfo` instead", "2.11.0")
-      def setTypeSignature[S <: Symbol](sym: S, tpe: Type): S = internal.reificationSupport.setInfo(sym, tpe)
-
-      /** @see [[BuildApi.FlagsRepr]] */
-      @deprecated("use `internal.reificationSupport.FlagsRepr` instead", "2.11.0")
-      def flagsFromBits(bits: Long): FlagSet = internal.reificationSupport.FlagsRepr(bits)
-
-      /** @see [[BuildApi.noSelfType]] */
-      @deprecated("use `noSelfType` instead", "2.11.0")
-      def emptyValDef: ValDef = noSelfType
-
-      /** @see [[BuildApi.mkThis]] */
-      @deprecated("use `internal.reificationSupport.mkThis` instead", "2.11.0")
-      def This(sym: Symbol): Tree = internal.reificationSupport.mkThis(sym)
-
-      /** @see [[BuildApi.mkSelect]] */
-      @deprecated("use `internal.reificationSupport.mkSelect` instead", "2.11.0")
-      def Select(qualifier: Tree, sym: Symbol): Select = internal.reificationSupport.mkSelect(qualifier, sym)
-
-      /** @see [[BuildApi.mkIdent]] */
-      @deprecated("use `internal.reificationSupport.mkIdent` instead", "2.11.0")
-      def Ident(sym: Symbol): Ident = internal.reificationSupport.mkIdent(sym)
-
-      /** @see [[BuildApi.mkTypeTree]] */
-      @deprecated("use `internal.reificationSupport.mkTypeTree` instead", "2.11.0")
-      def TypeTree(tp: Type): TypeTree = internal.reificationSupport.mkTypeTree(tp)
-    }
+    @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
+    implicit class CompatibleBuildApi(api: ReificationSupportApi)
 
     /** Scala 2.10 compatibility enrichments for Tree. */
-    implicit class CompatibleTree(tree: Tree) {
-      /** @see [[InternalApi.freeTerms]] */
-      @deprecated("use `internal.freeTerms` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def freeTerms: List[FreeTermSymbol] = internal.freeTerms(tree)
-
-      /** @see [[InternalApi.freeTypes]] */
-      @deprecated("use `internal.freeTerms` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def freeTypes: List[FreeTypeSymbol] = internal.freeTypes(tree)
-
-      /** @see [[InternalApi.substituteSymbols]] */
-      @deprecated("use `internal.substituteSymbols` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def substituteSymbols(from: List[Symbol], to: List[Symbol]): Tree = internal.substituteSymbols(tree, from, to)
-
-      /** @see [[InternalApi.substituteTypes]] */
-      @deprecated("use `internal.substituteTypes` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def substituteTypes(from: List[Symbol], to: List[Type]): Tree = internal.substituteTypes(tree, from, to)
-
-      /** @see [[InternalApi.substituteThis]] */
-      @deprecated("use `internal.substituteThis` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def substituteThis(clazz: Symbol, to: => Tree): Tree = internal.substituteThis(tree, clazz, to)
-    }
+    @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
+    implicit class CompatibleTree(tree: Tree)
 
     /** Scala 2.10 compatibility enrichments for Tree. */
-    implicit class CompatibleSymbol(symbol: Symbol) {
-      @deprecated("This API is unreliable. Use `isPrivateThis` or `isProtectedThis` instead", "2.11.0")
-      def isLocal: Boolean = symbol.asInstanceOf[scala.reflect.internal.Symbols#Symbol].isLocal
-
-      @deprecated("This API is unreliable. Use `overrides.nonEmpty` instead", "2.11.0")
-      def isOverride: Boolean = symbol.asInstanceOf[scala.reflect.internal.Symbols#Symbol].isOverride
-
-      /** @see [[InternalApi.isFreeTerm]] */
-      @deprecated("use `internal.isFreeTerm` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def isFreeTerm: Boolean = internal.isFreeTerm(symbol)
-
-      /** @see [[InternalApi.asFreeTerm]] */
-      @deprecated("use `internal.asFreeTerm` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def asFreeTerm: FreeTermSymbol = internal.asFreeTerm(symbol)
-
-      /** @see [[InternalApi.isFreeType]] */
-      @deprecated("use `internal.isFreeType` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def isFreeType: Boolean = internal.isFreeType(symbol)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.asFreeType` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def asFreeType: FreeTypeSymbol = internal.asFreeType(symbol)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.newTermSymbol` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def newTermSymbol(name: TermName, pos: Position = NoPosition, flags: FlagSet = NoFlags): TermSymbol = internal.newTermSymbol(symbol, name, pos, flags)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.newModuleAndClassSymbol` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def newModuleAndClassSymbol(name: Name, pos: Position = NoPosition, flags: FlagSet = NoFlags): (ModuleSymbol, ClassSymbol) = internal.newModuleAndClassSymbol(symbol, name, pos, flags)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.newMethodSymbol` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def newMethodSymbol(name: TermName, pos: Position = NoPosition, flags: FlagSet = NoFlags): MethodSymbol = internal.newMethodSymbol(symbol, name, pos, flags)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.newTypeSymbol` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def newTypeSymbol(name: TypeName, pos: Position = NoPosition, flags: FlagSet = NoFlags): TypeSymbol = internal.newTypeSymbol(symbol, name, pos, flags)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.newClassSymbol` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def newClassSymbol(name: TypeName, pos: Position = NoPosition, flags: FlagSet = NoFlags): ClassSymbol = internal.newClassSymbol(symbol, name, pos, flags)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.isErroneous` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def isErroneous: Boolean = internal.isErroneous(symbol)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.isSkolem` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def isSkolem: Boolean = internal.isSkolem(symbol)
-
-      /** @see [[InternalApi.asFreeType]] */
-      @deprecated("use `internal.deSkolemize` instead or import `internal.decorators._` for infix syntax", "2.11.0")
-      def deSkolemize: Symbol = internal.deSkolemize(symbol)
-    }
-
-    /** @see [[InternalApi.singleType]] */
-    @deprecated("use `internal.singleType` instead", "2.11.0")
-    def singleType(pre: Type, sym: Symbol): Type = internal.singleType(pre, sym)
-
-    /** @see [[InternalApi.refinedType]] */
-    @deprecated("use `internal.refinedType` instead", "2.11.0")
-    def refinedType(parents: List[Type], owner: Symbol, decls: Scope, pos: Position): Type = internal.refinedType(parents, owner, decls, pos)
-
-    /** @see [[InternalApi.refinedType]] */
-    @deprecated("use `internal.refinedType` instead", "2.11.0")
-    def refinedType(parents: List[Type], owner: Symbol): Type = internal.refinedType(parents, owner)
-
-    /** @see [[InternalApi.typeRef]] */
-    @deprecated("use `internal.typeRef` instead", "2.11.0")
-    def typeRef(pre: Type, sym: Symbol, args: List[Type]): Type = internal.typeRef(pre, sym, args)
-
-    /** @see [[InternalApi.intersectionType]] */
-    @deprecated("use `internal.intersectionType` instead", "2.11.0")
-    def intersectionType(tps: List[Type]): Type = internal.intersectionType(tps)
-
-    /** @see [[InternalApi.intersectionType]] */
-    @deprecated("use `internal.intersectionType` instead", "2.11.0")
-    def intersectionType(tps: List[Type], owner: Symbol): Type = internal.intersectionType(tps, owner)
-
-    /** @see [[InternalApi.polyType]] */
-    @deprecated("use `internal.polyType` instead", "2.11.0")
-    def polyType(tparams: List[Symbol], tpe: Type): Type = internal.polyType(tparams, tpe)
-
-    /** @see [[InternalApi.existentialAbstraction]] */
-    @deprecated("use `internal.existentialAbstraction` instead", "2.11.0")
-    def existentialAbstraction(tparams: List[Symbol], tpe0: Type): Type = internal.existentialAbstraction(tparams, tpe0)
+    @deprecated("compatibility with Scala 2.10 EOL", "2.13.0")
+    implicit class CompatibleSymbol(symbol: Symbol)
   }
 }
