@@ -22,10 +22,10 @@ abstract class Phase(val prev: Phase) {
   val id: Id = if (prev eq null) 0 else prev.id + 1
 
   /** New flags visible after this phase has completed */
-  def nextFlags: Long = 0l
+  def nextFlags: Long = 0L
 
   /** New flags visible once this phase has started */
-  def newFlags: Long = 0l
+  def newFlags: Long = 0L
 
   val fmask = (
     if (prev eq null) Flags.InitialFlags
