@@ -16,11 +16,15 @@ import scala.tools.nsc.Settings
 import scala.tools.nsc.reporters.StoreReporter
 
 trait StoreReporterDirectTest extends DirectTest {
-  lazy val storeReporter: StoreReporter = new scala.tools.nsc.reporters.StoreReporter()
+  private[this] var _storeReporter: StoreReporter = _
+  def storeReporter: StoreReporter = _storeReporter
 
   /** Discards all but the first message issued at a given position. */
-  def filteredInfos: Seq[storeReporter.Info] = storeReporter.infos.groupBy(_.pos).map(_._2.head).toList
+  def filteredInfos: Seq[StoreReporter.Info] = storeReporter.infos.groupBy(_.pos).map(_._2.head).toList
 
   /** Hook into [[scala.tools.partest.DirectTest]] to install the custom reporter */
-  override def reporter(settings: Settings) = storeReporter
+  override def reporter(settings: Settings) = {
+    _storeReporter = new StoreReporter(settings)
+    _storeReporter
+  }
 }
